/**
 * Layout
 * @typedef {Object} Layout
 *
 * @since 3.0.0
 *
 * @property {String} ID                  Layout unique identifier
 * @property {String} USER_ID             User identifier that created the layout
 * @property {String} ORGANIZATION_ID     Organization the user belongs to
 * @property {String} LABEL_TO_PREFERENCE Layout label
 * @property {String} WIDGET_ID           Widget identifier what the layout belongs to
 * @property {String} LAYOUT_CHANGES      Widget changes saved on the layout
 * @property {String} LAST_MODIFIED_DATE  Last date the layout was modified
 */

/**
 * Service to handle preferences storage
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}              ZhPreferences   Service to handle preferences storage
 * @param {TemplateManager}            templateManager Service to handle template
 * @param {EventAggregator}            eventAggregator Service to use project events
 * @param {ContainerMerger}            containerMerger Service to handle layout changes
 * @param {ZHPromise}                  ZHPromise       Service to handle promises
 * @param {EventEngine}                eventEngine     Service to use models events
 * @param {ZhUtil}                     ZhUtil          Service to provice utilities methods
 * @param {MessageService}             MessageService  Service to handle messages
 */
function ZhLayoutPreferences(ZhPreferences, templateManager, eventAggregator, containerMerger, ZHPromise, eventEngine, ZhUtil, MessageService){

    var originalLayout = {
        'ID': 'ORIGINAL',
        'USER_ID': 0,
        'ORGANIZATION_ID': 0,
        'LAYOUT_CHANGES': '{}',
        'LABEL_TO_PREFERENCE': 'Original'
    };

    function shouldShowLayoutAction(widget){
        return Util.get(widget, ['floatingControl', 'customizationAction', 'layout']) !== false;
    }

    eventAggregator.subscribe('beforeProjectStartup', function(){
        ZhPreferences.registerRepository('zh-layout-preferences#layout');

        templateManager.project.customFloatingActions.push({
            'name': 'Layout Action',
            'label': 'Layout',
            'type': 'layout',
            'order': 20,
            'icon': 'layout',
            'template': 'zh-layout-preferences#floating-layout-card.html',
            'expandable': true,
            'isVisible': shouldShowLayoutAction
        });
    });

    eventEngine.registerEvent('LayoutBeforeApply', 'beforeLayoutApply');
    eventEngine.registerEvent('LayoutAfterApply', 'afterLayoutApply');

    var layoutRegisteredDataSources = {};

    /**
     * Register layouts datasource
     *
     * @since 3.0.0
     *
     * @param {String} widgetID         Widget identifier
     * @param {Object} datasource       Datasource that contains the layouts
     *
     */
    this.registerLayoutDataSource = function(widgetID, dataSource){
        if(widgetID){
            layoutRegisteredDataSources[widgetID] = dataSource;
        }

        return layoutRegisteredDataSources;
    };

    /**
     * Update dataSource with local saved layouts
     *
     * @since 3.0.0
     *
     * @param  {String}     widgetID   Widget identifier
     * @param  {DataSource} dataSource DataSource that contains the loaded layouts
     *
     * @return {Promise}               Promise resolved after the layouts are loaded into the dataSource
     */
    this.updateLayouts = function(widgetID, dataSource){
        if(!dataSource){
            dataSource = layoutRegisteredDataSources[widgetID];
        }

        return this.getWidgetLayouts(widgetID).then(function(layouts){
            return layouts;
        }).then(function(layouts){
            return this.addIsDefaultLayoutToLayouts(layouts, widgetID).then(function(layouts){
                dataSource.data = layouts;
            });
        }.bind(this));
    };

    /**
     * Add property to identify if the layout is selected as default
     *
     * @since 3.0.0
     *
     * @param {Layout[]} layouts  List of layouts to update the default layout property
     * @param {String}   widgetID Widget to get the default properties
     */
    this.addIsDefaultLayoutToLayouts = function(layouts, widgetID){
        return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){
            var defaultLayout = defaultPreferences.DEFAULT_LAYOUT;
            return layouts.map(addIsDefaultLayoutToLayout.bind(this, defaultLayout));
        });
    };

    function addIsDefaultLayoutToLayout(defaultLayout, layout){
        layout.DEFAULT_LAYOUT = layout.ID === defaultLayout;
        return layout;
    }

    /**
     * Retrieve local cached layouts for specified widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetID   Widget identifier
     *
     * @return {Promise<Layout[]>} Layout preferences
     */
    this.getWidgetLayouts = function(widgetID){
        return ZhPreferences.getWidgetPreferences('layout', widgetID);
    };

    /**
     * Retrieve local cached layouts
     *
     * @since 3.0.0
     *
     * @param  {Query} query       Conditions to filter the preferences
     *
     * @return {Promise<Layout[]>} Layout preferences
     */
    this.getLayouts = function(query){
        return ZhPreferences.getPreferences('layout', query);
    };

    /**
     * Persist layouts
     *
     * @since 3.0.0
     *
     * @param  {Layout[]} layouts  Layouts to persist
     *
     * @return {Promise<Layout[]>} List of persisted layouts
     */
    this.saveLayouts = function(layouts){
        var cloneLayouts = layouts.map(function(layout){ return ZhUtil.clearRow(layout); });
        return ZhPreferences.savePreferences('layout', cloneLayouts).then(function(response){
            cloneLayouts.forEach(function(clone, index){
                layouts[index].ID = clone.ID;
                layouts[index].__is_new = clone.__is_new;
                layouts[index].changes = clone.changed;
            });

            return ZHPromise.when(response);
        });
    };

    /**
     * Delete layouts
     *
     * @since 3.0.0
     *
     * @param  {Layout[]} layouts Layouts to delete
     *
     * @return {Promise<Layout[]>} List of deleted layouts
     */
    this.deleteLayouts = function(layouts){
        var cloneLayouts = layouts.map(function(layout){ return ZhUtil.clearRow(layout); });
        return ZhPreferences.deletePreferences('layout', cloneLayouts);
    };

    function triggerBeforeLayoutApplyEvent(widget, layout){
        if(widget.beforeLayoutApply){
            widget.beforeLayoutApply({
                'layout': layout
            });
        }
    }

    function triggerAfterLayoutApplyEvent(widget, layout){
        if(widget.afterLayoutApply){
            widget.afterLayoutApply({
                'layout': layout
            });
        }
    }

    var appliedPreference = {ID: 'ORIGINAL'};

    /**
     * Apply original layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget Widget to apply the original layout on
     *
     * @return {Promise}       Promise resolved after the original layout is applied
     */
    this.applyOriginalLayout = function(widget){
        triggerBeforeLayoutApplyEvent(widget, originalLayout);

        containerMerger.restore(widget, 'ORIGINAL');
        widget.redraw();
        widget.setAppliedLayout(originalLayout);
        appliedPreference = originalLayout;

        triggerAfterLayoutApplyEvent(widget, originalLayout);

        return ZHPromise.when();
    };

    /**
     * Apply layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget     Widget to apply the layout on
     * @param  {Layout} preference Layout preference to be applied
     *
     * @return {Promise}           Promise that is resolved after the layout is applied
     */
    this.applyLayout = function(widget, preference){
        triggerBeforeLayoutApplyEvent(widget, preference);

        if(widget.backupPDFOrientation === undefined){
            widget.backupPDFOrientation = widget.reportOrientation || 'PORTRAIT';
        }
        widget.reportOrientation = preference.PDF_REPORT_ORIENTATION;

        var layoutChanges = prepareLayout(JSON.parse(preference.LAYOUT_CHANGES));
        containerMerger.merge(templateManager.container, layoutChanges, appliedPreference.ID);

        widget.redraw(true);

        widget.setAppliedLayout(preference);
        appliedPreference = preference;

        triggerAfterLayoutApplyEvent(widget, preference);

        return ZHPromise.when();
    };

    var propertyToBeVerified = {
        container: ['widgets'],
        widget: ['widgets', 'fields']
    };

    function prepareLayout(objToBeMerged, baseObject){
        baseObject = baseObject || templateManager.container;
        var childProperties = [];

        if(baseObject instanceof Container){
            childProperties = propertyToBeVerified.container;
        } else if(baseObject instanceof Widget){
            childProperties = propertyToBeVerified.widget;
        }

        childProperties.forEach(function prepareChildLayout(propertyName){
            var childBaseObj = baseObject[propertyName];
            var childObjToBeMerged = objToBeMerged[propertyName];
            if(childObjToBeMerged){
                if(childBaseObj && childBaseObj.length){
                    var existentIds = childBaseObj.map(function returnObjId(obj){
                        return obj.id;
                    });
                    objToBeMerged[propertyName] = childObjToBeMerged.filter(function filterChilds(obj){
                        var objIndex = existentIds.indexOf(obj.id);
                        if(objIndex != -1){
                            prepareLayout(obj, childBaseObj[objIndex]);
                        }
                        return ~objIndex;
                    });
                } else {
                    objToBeMerged[propertyName] = [];
                }
            }
        });

        return objToBeMerged;
    }

    /**
     * Export layout file
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout  Layout to export
     *
     * @return {Promise<Layout>} Layout exported
     */
    this.downloadPreferenceFile = function(layout){
        var fileName = layout.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/ig,'').replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index){
            if(+match === 0) return "";
            return index == 0 ? match.toLowerCase() : match.toUpperCase();
        }) + '.layout';

        var layoutToExport = {
            'ID': layout.ID,
            'LABEL_TO_PREFERENCE': layout.LABEL_TO_PREFERENCE,
            'WIDGET_ID': layout.WIDGET_ID,
            'PDF_REPORT_ORIENTATION': layout.PDF_REPORT_ORIENTATION,
            'LAYOUT_CHANGES': JSON.parse(layout.LAYOUT_CHANGES),
            'LAST_MODIFIED_DATE': layout.LAST_MODIFIED_DATE
        };

        return ZhPreferences.downloadPreferenceFile(layoutToExport, fileName);
    };

    /**
     * Validade layout to import
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout              Layout to import
     * @param  {Widget} configWidget        Widget to validate
     * @param  {Object[]} layoutDataSource  Existing layouts data (optional)
     *
     * @return {Boolean} Layout valid
     */
    this.validateLayoutImport = function(layout, configWidget, layoutDataSource){
        layoutDataSource = layoutDataSource || configWidget.dataSource.data;

        if(configWidget.listWidget.id != layout.WIDGET_ID){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_widget_id');
            return false;
        }

        var layoutsWithSameId = layoutDataSource.filter(function(row){
            return row.ID == layout.ID;
        });

        if(layoutsWithSameId.length > 0){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_id_violation');
            return false;
        }

        var layoutsWithSameLabel = layoutDataSource.filter(function(row){
            return row.LABEL_TO_PREFERENCE == layout.LABEL_TO_PREFERENCE;
        });

        if(layoutsWithSameLabel.length > 0){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_name_violation');
            return false;
        }

        return true;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutPreferences', ZhLayoutPreferences);
});